/*
 * RSA signature key generation
 * Copyright (C) 1999  Henry Spencer.
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * RCSID $Id: rsasigkey.c,v 1.1 1999/12/14 21:23:51 henry Exp $
 */

#include <stdio.h>
#include <limits.h>
#include <errno.h>
#include <string.h>
#include <assert.h>
#include <getopt.h>
#include <freeswan.h>
#include "gmp.h"

#ifndef DEVICE
#define	DEVICE	"/dev/random"
#endif
#ifndef MAXBITS
#define	MAXBITS	4096
#endif

char usage[] = "Usage: rsasigkey [--verbose] [--random device] nbits";
struct option opts[] = {
	"verbose",	0,	NULL,	'v',
	"random",	1,	NULL,	'r',
	"help",		0,	NULL,	'h',
	"version",	0,	NULL,	'V',
	0,		0,	NULL,	0,
};
int verbose = 0;		/* narrate the action? */
char *device = DEVICE;		/* where to get randomness */

char me[] = "ipsec rsasigkey";	/* for messages */

/* forwards */
int rsasigkey(int nbits);
void initprime(mpz_t var, int nbits, int eval);
void initrandom(mpz_t var, int nbits);
void getrandom(size_t nbytes, char *buf);
char *hexout(mpz_t var);

/*
 - main - mostly argument parsing
 */
main(argc, argv)
int argc;
char *argv[];
{
	int opt;
	extern int optind;
	extern char *optarg;
	int errflg = 0;
	int nbits;
	int i;

	while ((opt = getopt_long(argc, argv, "", opts, NULL)) != EOF)
		switch (opt) {
		case 'v':	/* verbose description */
			verbose = 1;
			break;
		case 'r':	/* nonstandard /dev/random */
			device = optarg;
			break;
		case 'h':	/* help */
			printf("%s\n", usage);
			exit(0);
			break;
		case 'V':	/* version */
			printf("1\n");
			exit(0);
			break;
		case '?':
		default:
			errflg = 1;
			break;
		}
	if (errflg || optind != argc-1) {
		fprintf(stderr, "%s\n", usage);
		exit(2);
	}

	nbits = atoi(argv[optind]);
	if (nbits <= 0) {
		fprintf(stderr, "%s: invalid bit count (%d)\n", me, nbits);
		exit(1);
	}
	if (nbits > MAXBITS) {
		fprintf(stderr, "%s: overlarge bit count (max %d)\n", me,
								MAXBITS);
		exit(1);
	}
	if (nbits % (CHAR_BIT*2) != 0) {
		fprintf(stderr, "%s: bit count (%d) not multiple of %d\n", me,
						nbits, (int)CHAR_BIT*2);
		exit(1);
	}

	i = 1;
	while (rsasigkey(nbits) < 0) {
		i++;
		if (verbose)
			fprintf(stderr, "try #%d:\n", i);
	}
	exit(0);
}

/*
 - rsasigkey - generate an RSA signature key
 * We take e to be 3, without discussion.  That would not be wise if these
 * keys were to be used for encryption, but for signatures there are some
 * real speed advantages.
 */
int				/* 0 worked, <0 must try again */
rsasigkey(nbits)
int nbits;
{
	mpz_t p;
	mpz_t q;
	mpz_t n;
	mpz_t e;
#	define	E	3
	mpz_t d;
	mpz_t m;			/* (p-1)*(q-1) */
	mpz_t t;			/* temporary */
	mpz_t exp1;
	mpz_t exp2;
	mpz_t coeff;
	char *hexp;
	size_t len;
	static char out[MAXBITS/6 + 10];	/* 10 = a bit of slop */
	const char *err;

	/* the easy stuff */
	if (verbose)
		fprintf(stderr, "computing primes and n\n");
	initprime(p, nbits/2, E);
	initprime(q, nbits/2, E);
	mpz_init(t);
	if (mpz_cmp(p, q) < 0) {	/* p to be the larger of the primes */
		mpz_set(t, p);
		mpz_set(p, q);
		mpz_set(q, t);
	}
	mpz_init(n);
	mpz_mul(n, p, q);		/* n = p*q */
	mpz_init_set_ui(e, E);

	/* the modulus */
	if (verbose)
		fprintf(stderr, "computing and checking (p-1)*(q-1)\n");
	mpz_init_set(m, p);
	mpz_sub_ui(m, m, 1);
	mpz_set(t, q);
	mpz_sub_ui(t, t, 1);
	mpz_mul(m, m, t);		/* m = (p-1)*(q-1) */
	mpz_gcd(t, m, e);
	if (mpz_cmp_ui(t, 1) != 0) {
		if (verbose)
			fprintf(stderr, "oops, m and e not relatively prime, "
							"must start over\n");
		mpz_clear(p);
		mpz_clear(q);
		mpz_clear(n);
		mpz_clear(e);
		mpz_clear(m);
		mpz_clear(t);
		return -1;
	}

	/* the decryption key */
	if (verbose)
		fprintf(stderr, "computing d\n");
	mpz_init(d);
	if (!mpz_invert(d, e, m)) {
		if (verbose)
			fprintf(stderr, "oops, e has no inverse, "
							"must start over\n");
		mpz_clear(p);
		mpz_clear(q);
		mpz_clear(n);
		mpz_clear(e);
		mpz_clear(m);
		mpz_clear(t);
		mpz_clear(d);
		return -1;
	}
	if (mpz_cmp_ui(d, 0) < 0)
		mpz_add(d, d, m);

	/* the speedup hacks */
	mpz_init(exp1);
	mpz_sub_ui(t, p, 1);
	mpz_mod(exp1, d, t);		/* exp1 = d mod p-1 */
	mpz_init(exp2);
	mpz_sub_ui(t, q, 1);
	mpz_mod(exp2, d, t);		/* exp2 = d mod q-1 */
	mpz_init(coeff);
	mpz_invert(coeff, q, p);	/* coeff = q^-1 mod p */
	if (mpz_cmp_ui(coeff, 0) < 0)
		mpz_add(coeff, coeff, p);

	/* and the output */
	hexp = hexout(n);
	printf("\t#pubkey=0x01%02x%s\n", E, hexp+2);	/* RFC 2537 format */
	printf("\tModulus: %s\n", hexp);
	printf("\tPublicExponent: %s\n", hexout(e));
	printf("\tPrivateExponent: %s\n", hexout(d));
	printf("\tPrime1: %s\n", hexout(p));
	printf("\tPrime2: %s\n", hexout(q));
	printf("\tExponent1: %s\n", hexout(exp1));
	printf("\tExponent2: %s\n", hexout(exp2));
	printf("\tCoefficient: %s\n", hexout(coeff));
	return(0);
}

/*
 - initprime - initialize an mpz_t to a random prime of specified size
 * Going up by 2 rather than 1 would be better, but that means having to
 * ensure that the first value is odd, and that's too much trouble; the
 * prime checker efficiently rejects even numbers anyway.  Efficiency tweak:
 * we reject primes that are 1 higher than a multiple of e, since they will
 * make the modulus not relatively prime to e.
 */
void
initprime(var, nbits, eval)
mpz_t var;
int nbits;			/* known to be a multiple of CHAR_BIT */
int eval;			/* value of e; 0 means don't bother */
{
	unsigned long off;

	initrandom(var, nbits);

	if (verbose)
		fprintf(stderr, "looking for a prime starting there...\n");

	off = 1;
	while (!mpz_probab_prime_p(var, 30) ||
				(eval != 0 && mpz_fdiv_ui(var, eval) == 1)) {
		mpz_add_ui(var, var, 1);
		off++;
	}

	if (verbose)		/* don't count the even numbers! */
		fprintf(stderr, "found it after %lu tries\n", off/2);
}

/*
 - initrandom - initialize an mpz_t to a specified number of random bits
 * Going via hex is a bit strange, but it's the best route GMP gives us.
 */
void
initrandom(var, nbits)
mpz_t var;
int nbits;			/* known to be a multiple of CHAR_BIT */
{
	size_t nbytes = (size_t)(nbits / CHAR_BIT);
	static char bitbuf[MAXBITS/CHAR_BIT];
	static char hexbuf[2 + MAXBITS/4 + 1];
	size_t hsize = sizeof(hexbuf);

	assert(nbytes <= sizeof(bitbuf));
	getrandom(nbytes, bitbuf);
	if (bytestoa(bitbuf, nbytes, 'x', hexbuf, hsize) > hsize) {
		fprintf(stderr, "%s: can't-happen buffer overflow\n", me);
		exit(1);
	}
	if (mpz_init_set_str(var, hexbuf, 0) < 0) {
		fprintf(stderr, "%s: can't-happen hex conversion error\n", me);
		exit(1);
	}
}

/*
 - getrandom - get some random bytes
 */
void
getrandom(nbytes, buf)
size_t nbytes;
char *buf;			/* known to be big enough */
{
	size_t ndone;
	int dev;
	size_t got;

	dev = open(device, 0);
	if (dev < 0) {
		fprintf(stderr, "%s: could not open %s (%s)\n", me,
						device, strerror(errno));
		exit(1);
	}

	ndone = 0;
	if (verbose)
		fprintf(stderr, "getting %d random bytes from %s\n", nbytes,
								device);
	while (ndone < nbytes) {
		got = read(dev, buf + ndone, nbytes - ndone);
		if (got < 0) {
			fprintf(stderr, "%s: read error on %s (%s)\n", me,
						device, strerror(errno));
			exit(1);
		}
		if (got == 0) {
			fprintf(stderr, "%s: eof on %s!?!\n", me, device);
			exit(1);
		}
		ndone += got;
	}

	close(dev);
}

/*
 - hexout - prepare hex output, guaranteeing even number of digits
 */
char *				/* pointer to static buffer (ick) */
hexout(var)
mpz_t var;
{
	static char hexbuf[3 + MAXBITS/4 + 1];
	char *hexp;

	mpz_get_str(hexbuf+3, 16, var);
	if (strlen(hexbuf+3)%2 == 0)	/* even number of hex digits */
		hexp = hexbuf+1;
	else {				/* odd, must pad */
		hexp = hexbuf;
		hexp[2] = '0';
	}
	hexp[0] = '0';
	hexp[1] = 'x';

	return hexp;
}
